Attribute VB_Name = "mCallWindowProc"
'---------------------------------------------------------------------------------------
' Module      : mCallWindowProc
' DateTime    : 24/10/2007 15:05
' Author      : Cobein
' Mail        : cobein27@hotmail.com
' Purpose     : Implements CallWindowProc using multiple parameters
' Requirements: None
' Distribution: You can freely use this code in your own
'               applications, but you may not reproduce
'               or publish this code on any web site,
'               online service, or distribute as source
'               on any media without express permission.
'---------------------------------------------------------------------------------------
Option Explicit
Option Base 0

Private Declare Function LoadLibraryA Lib "kernel32" (ByVal lpLibFileName As String) As Long
Private Declare Function GetProcAddress Lib "kernel32" (ByVal hModule As Long, ByVal lpProcName As String) As Long
Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function FreeLibrary Lib "kernel32" (ByVal hLibModule As Long) As Long
Private Declare Sub RtlMoveMemory Lib "kernel32" (lpDest As Any, lpSource As Any, ByVal cBytes As Long)
Private Declare Sub FillMemory Lib "kernel32.dll" Alias "RtlFillMemory" (Destination As Any, ByVal Length As Long, ByVal Fill As Byte)

Private m_lLib          As Long
Private m_lProc         As Long
Private m_lvParams()    As Long
Private m_lCallPtr      As Long
Private m_bvCode()      As Byte

Public Function UnloadLib()
    Call FreeLibrary(m_lLib)
End Function

Public Function LoadFunction( _
       ByVal sLibrary As String, _
       ByVal sProcedure As String) As Boolean
    
    Call FreeLibrary(m_lLib)
    m_lLib = LoadLibraryA(sLibrary)
    If m_lLib = 0 Then Exit Function
    
    m_lProc = GetProcAddress(m_lLib, sProcedure)
    If m_lLib = 0 Then
        Call FreeLibrary(m_lLib)
        Exit Function
    End If
    LoadFunction = True
    
End Function

Public Function CallFunction(ParamArray FuncParams()) As Long
    Dim i           As Long
    Dim lSize       As Long
    
    If m_lProc = 0 Then Exit Function
    
    lSize = UBound(FuncParams) + 1
    ReDim m_lvParams(lSize)
    ReDim m_bvCode(18 + 32 + 6 * lSize)
    
    For i = 0 To lSize - 1
        If Not IsNumeric(FuncParams(i)) Then Exit Function
        m_lvParams(i + 1) = CLng(FuncParams(i))
    Next i
    
    CallFunction = CallWindowProc(PrepareCode, 0, 0, 0, 0)
    
End Function

Private Function PrepareCode() As Long
    Dim i           As Long
    Dim lPtr        As Long

    lPtr = VarPtr(m_bvCode(0))
    PrepareCode = GetAlignedCodeStart(lPtr)
    m_lCallPtr = PrepareCode - lPtr

    Call FillMemory(m_bvCode(i), m_lCallPtr - 1, &HCC)
    
    Call AddByteToCode(&H58)
    Call AddLongToCode(&H59595959)
    Call AddByteToCode(&H50)
    
    For i = UBound(m_lvParams) To 1 Step -1
        Call AddByteToCode(&H68)
        Call AddLongToCode(m_lvParams(i))
    Next
    
    Call AddCallToCode(m_lProc)
    Call AddByteToCode(&HC3)
    Call AddByteToCode(&HCC)
    
End Function

Private Function GetAlignedCodeStart(ByVal lAddress As Long) As Long
    Dim lRet As Long
    lRet = (15 - (lAddress - 1) Mod 16)
    lRet = IIf(lRet = 0, lAddress + 16, lAddress + lRet)
    GetAlignedCodeStart = lRet
End Function

Private Sub AddCallToCode(ByVal lAddress As Long)
    Dim lPtr As Long
    Call AddByteToCode(&HE8)
    lPtr = (lAddress - VarPtr(m_bvCode(m_lCallPtr)) - 4)
    Call AddLongToCode(lPtr)
End Sub

Private Sub AddLongToCode(ByVal lVal As Long)
    Dim bvLng(3) As Byte
    Call RtlMoveMemory(bvLng(0), lVal, &H4)
    Call AddByteToCode(bvLng(0))
    Call AddByteToCode(bvLng(1))
    Call AddByteToCode(bvLng(2))
    Call AddByteToCode(bvLng(3))
End Sub

Private Sub AddByteToCode(ByVal bVal As Byte)
    m_bvCode(m_lCallPtr) = bVal
    m_lCallPtr = m_lCallPtr + 1
End Sub

